@@ -29,16 +29,6 @@ class FitterTest(unittest.TestCase):
         cls.heart_disease = Table('heart_disease')
         cls.housing = Table('housing')
 
-    def test_throws_if_fits_property_is_invalid(self):
-        """The `__fits__` attribute must be an instance of `LearnerTypes`"""
-        with self.assertRaises(Exception):
-            class DummyFitter(Fitter):
-                name = 'dummy'
-                __fits__ = (DummyClassificationLearner, DummyRegressionLearner)
-
-            fitter = DummyFitter()
-            fitter(self.heart_disease)
-
     def test_dispatches_to_correct_learner(self):
         """Based on the input data, it should dispatch the fitting process to
         the appropriate learner"""
@@ -102,15 +92,6 @@ class FitterTest(unittest.TestCase):
         except TypeError:
             self.fail('Fitter did not properly distribute params to learners')
 
-    def test_error_for_data_type_with_no_learner(self):
-        """If we attempt to define a fitter which only handles one data type
-        it makes more sense to simply use a Learner."""
-        with self.assertRaises(AssertionError):
-            class DummyFitter(Fitter):
-                name = 'dummy'
-                __fits__ = {'classification': None,
-                            'regression': DummyRegressionLearner}
-
     def test_correctly_sets_preprocessors_on_learner(self):
         """Fitters have to be able to pass the `use_default_preprocessors` and
         preprocessors down to individual learners"""
@@ -152,3 +133,43 @@ class FitterTest(unittest.TestCase):
         pp_data = fitter.preprocess(self.heart_disease)
         self.assertTrue(not any(
             isinstance(v, ContinuousVariable) for v in pp_data.domain.variables))
+
+    def test_default_kwargs_with_change_kwargs(self):
+        """Fallback to default args in case specialized params not specified.
+        """
+        class DummyClassificationLearner(LearnerClassification):
+            def __init__(self, param='classification_default', **_):
+                super().__init__()
+                self.param = param
+
+            def fit_storage(self, data):
+                return DummyModel(self.param)
+
+        class DummyRegressionLearner(LearnerRegression):
+            def __init__(self, param='regression_default', **_):
+                super().__init__()
+                self.param = param
+
+            def fit_storage(self, data):
+                return DummyModel(self.param)
+
+        class DummyModel:
+            def __init__(self, param):
+                self.param = param
+
+        class DummyFitter(Fitter):
+            __fits__ = {'classification': DummyClassificationLearner,
+                        'regression': DummyRegressionLearner}
+
+            def _change_kwargs(self, kwargs, problem_type):
+                if problem_type == self.CLASSIFICATION:
+                    kwargs['param'] = kwargs.get('classification_param')
+                else:
+                    kwargs['param'] = kwargs.get('regression_param')
+                return kwargs
+
+        learner = DummyFitter()
+        iris, housing = Table('iris')[:5], Table('housing')[:5]
+        self.assertEqual(learner(iris).param, 'classification_default')
+        self.assertEqual(learner(housing).param, 'regression_default')
+
