@@ -5,13 +5,14 @@ from AnyQt.QtWidgets import QFormLayout, QApplication
 from AnyQt.QtCore import Qt
 
 import numpy as np
-import sklearn.cross_validation as skl_cross_validation
+import sklearn.model_selection as skl
 
 from Orange.widgets import widget, gui
 from Orange.widgets.settings import Setting
 from Orange.data import Table
 from Orange.data.sql.table import SqlTable
 from Orange.widgets.widget import Msg, OWWidget
+from Orange.util import Reprable
 
 
 class OWDataSampler(OWWidget):
@@ -109,7 +110,7 @@ class OWDataSampler(OWWidget):
             addToLayout=False, callback=self.fold_changed)
         form.addRow("Selected fold:", self.selected_fold_spin)
 
-        gui.appendRadioButton(sampling, "Boostrap")
+        gui.appendRadioButton(sampling, "Bootstrap")
 
         self.sql_box = gui.vBox(self.controlArea, "Sampling Type")
         sampling = gui.radioButtons(self.sql_box, self, "sampling_type",
@@ -258,28 +259,26 @@ class OWDataSampler(OWWidget):
                       type(self.data) == Table and
                       self.data.domain.has_discrete_class)
         try:
-            self.sample(data_length, size, stratified)
+            self.indices = self.sample(data_length, size, stratified)
         except ValueError as ex:
             self.Warning.could_not_stratify(str(ex))
-            self.sample(data_length, size, stratified=False)
+            self.indices = self.sample(data_length, size, stratified=False)
 
     def sample(self, data_length, size, stratified):
         rnd = self.RandomSeed if self.use_seed else None
         if self.sampling_type == self.FixedSize:
-            self.indices = sample_random_n(
-                self.data, size,
-                stratified=stratified, replace=self.replacement,
+            self.indice_gen = SampleRandomN(
+                size, stratified=stratified, replace=self.replacement,
                 random_state=rnd)
         elif self.sampling_type == self.FixedProportion:
-            self.indices = sample_random_p(
-                self.data, self.sampleSizePercentage / 100,
-                stratified=stratified, random_state=rnd)
+            self.indice_gen = SampleRandomP(
+                self.sampleSizePercentage / 100, stratified=stratified, random_state=rnd)
         elif self.sampling_type == self.Bootstrap:
-            self.indices = sample_bootstrap(data_length, random_state=rnd)
+            self.indice_gen = SampleBootstrap(data_length, random_state=rnd)
         else:
-            self.indices = sample_fold_indices(
-                self.data, self.number_of_folds, stratified=stratified,
-                random_state=rnd)
+            self.indice_gen = SampleFoldIndices(
+                self.number_of_folds, stratified=stratified, random_state=rnd)
+        return self.indice_gen(self.data)
 
     def send_report(self):
         if self.sampling_type == self.FixedProportion:
@@ -312,62 +311,101 @@ class OWDataSampler(OWWidget):
         self.report_items(items)
 
 
-def sample_fold_indices(table, folds=10, stratified=False, random_state=None):
-    """
-    :param Orange.data.Table table:
-    :param int folds: Number of folds
-    :param bool stratified: Return stratified indices (if applicable).
-    :param Random random_state:
-    :rval tuple-of-arrays: A tuple of array indices one for each fold.
-    """
-    if stratified and table.domain.has_discrete_class:
-        # XXX: StratifiedKFold does not support random_state
-        ind = skl_cross_validation.StratifiedKFold(
-            table.Y.ravel(), folds, random_state=random_state)
-    else:
-        ind = skl_cross_validation.KFold(
-            len(table), folds, shuffle=True, random_state=random_state)
-    return tuple(ind)
+class SampleFoldIndices(Reprable):
+    def __init__(self, folds=10, stratified=False, random_state=None):
+        """Samples data based on a number of folds.
 
+        Args:
+            folds (int): Number of folds
+            stratified (bool): Return stratified indices (if applicable).
+            random_state (Random): An initial state for replicable random behavior
 
-def sample_random_n(table, n, stratified=False, replace=False,
-                    random_state=None):
-    if replace:
-        if random_state is None:
-            rgen = np.random
+        Returns:
+            tuple-of-arrays: A tuple of array indices one for each fold.
+
+        """
+        self.folds = folds
+        self.stratified = stratified
+        self.random_state = random_state
+
+    def __call__(self, table):
+        if self.stratified and table.domain.has_discrete_class:
+            splitter = skl.StratifiedKFold(
+                self.folds, random_state=self.random_state)
+            splitter.get_n_splits(table.X, table.Y)
+            ind = splitter.split(table.X, table.Y)
         else:
-            rgen = np.random.mtrand.RandomState(random_state)
-        sample = rgen.randint(len(table), size=n)
-        o = np.ones(len(table))
-        o[sample] = 0
-        others = np.nonzero(o)[0]
-        return others, sample
-    if stratified and table.domain.has_discrete_class:
-        test_size = max(len(table.domain.class_var.values), n)
-        ind = skl_cross_validation.StratifiedShuffleSplit(
-            table.Y.ravel(), n_iter=1,
-            test_size=test_size, train_size=len(table) - test_size,
-            random_state=random_state)
-    else:
-        ind = skl_cross_validation.ShuffleSplit(
-            len(table), n_iter=1,
-            test_size=n, random_state=random_state)
-    return next(iter(ind))
+            splitter = skl.KFold(
+                self.folds, random_state=self.random_state)
+            splitter.get_n_splits(table)
+            ind = splitter.split(table)
+        return tuple(ind)
 
 
-def sample_random_p(table, p, stratified=False, random_state=None):
-    n = int(math.ceil(len(table) * p))
-    return sample_random_n(table, n, stratified, False, random_state)
+class SampleRandomN(Reprable):
+    def __init__(self, n=0, stratified=False, replace=False,
+                    random_state=None):
+        self.n = n
+        self.stratified = stratified
+        self.replace = replace
+        self.random_state = random_state
+
+    def __call__(self, table):
+        if self.replace:
+            rgen = np.random.RandomState(self.random_state)
+            sample = rgen.random_integers(0, len(table) - 1, self.n)
+            o = np.ones(len(table))
+            o[sample] = 0
+            others = np.nonzero(o)[0]
+            return others, sample
+        if self.stratified and table.domain.has_discrete_class:
+            test_size = max(len(table.domain.class_var.values), self.n)
+            splitter = skl.StratifiedShuffleSplit(
+                n_splits=1, test_size=test_size,
+                train_size=len(table) - test_size,
+                random_state=self.random_state)
+            splitter.get_n_splits(table.X, table.Y)
+            ind = splitter.split(table.X, table.Y)
+        else:
+            splitter = skl.ShuffleSplit(
+                n_splits=1, test_size=self.n, random_state=self.random_state)
+            splitter.get_n_splits(table)
+            ind = splitter.split(table)
+        return next(iter(ind))
 
 
-def sample_bootstrap(size, random_state=None):
-    rgen = np.random.RandomState(random_state)
-    sample = rgen.randint(0, size, size)
-    sample.sort()  # not needed for the code below, just for the user
-    insample = np.ones((size,), dtype=np.bool)
-    insample[sample] = False
-    remaining = np.flatnonzero(insample)
-    return remaining, sample
+class SampleRandomP(Reprable):
+    def __init__(self, p=0, stratified=False, random_state=None):
+        self.p = p
+        self.stratified = stratified
+        self.random_state = random_state
+
+    def __call__(self, table):
+        n = int(math.ceil(len(table) * self.p))
+        return SampleRandomN(n, self.stratified,
+                             random_state=self.random_state)(table)
+
+
+class SampleBootstrap(Reprable):
+    def __init__(self, size=0, random_state=None):
+        self.size = size
+        self.random_state = random_state
+
+    def __call__(self, table=None):
+        """Bootstrap indices
+
+        Args:
+            table: Not used (but part of the signature)
+        Returns:
+            tuple (out_of_sample, sample) indices
+        """
+        rgen = np.random.RandomState(self.random_state)
+        sample = rgen.randint(0, self.size, self.size)
+        sample.sort()  # not needed for the code below, just for the user
+        insample = np.ones((self.size,), dtype=np.bool)
+        insample[sample] = False
+        remaining = np.flatnonzero(insample)
+        return remaining, sample
 
 
 def test_main():
