@@ -8,14 +8,13 @@ A widget for manual editing of a domain's attributes.
 import unicodedata
 from itertools import chain
 
-from PyQt4 import QtGui
-from PyQt4.QtGui import (
-    QWidget, QListView, QTreeView, QStandardItemModel, QStandardItem,
-    QVBoxLayout, QHBoxLayout, QFormLayout, QToolButton, QLineEdit,
-    QAction, QKeySequence
+from AnyQt.QtWidgets import (
+    QWidget, QListView, QTreeView, QVBoxLayout, QHBoxLayout, QFormLayout,
+    QToolButton, QLineEdit, QAction, QStackedWidget
 )
-from PyQt4.QtCore import Qt, QSize
-from PyQt4.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
+from AnyQt.QtGui import QStandardItemModel, QStandardItem, QKeySequence
+from AnyQt.QtCore import Qt, QSize
+from AnyQt.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
 
 import Orange.data
 import Orange.preprocess.transformation
@@ -37,7 +36,7 @@ def get_qualified(module, name):
     return getattr(module, name)
 
 
-def variable_description(var):
+def variable_description(var, skip_attributes=False):
     """Return a variable descriptor.
 
     A descriptor is a hashable tuple which should uniquely define
@@ -46,18 +45,21 @@ def variable_description(var):
 
     """
     var_type = type(var)
+    attributes = ()
+    if not skip_attributes:
+        attributes = tuple(sorted(var.attributes.items()))
     if var.is_discrete:
         return (var_type.__module__,
                 var_type.__name__,
                 var.name,
                 (("values", tuple(var.values)),),
-                tuple(sorted(var.attributes.items())))
+                attributes)
     else:
         return (var_type.__module__,
                 var_type.__name__,
                 var.name,
                 (),
-                tuple(sorted(var.attributes.items())))
+                attributes)
 
 
 def variable_from_description(description, compute_value=None):
@@ -68,7 +70,7 @@ def variable_from_description(description, compute_value=None):
     module, type_name, name, kwargs, attrs = description
     try:
         constructor = get_qualified(module, type_name)
-    except (ImportError, AttributeError) as ex:
+    except (ImportError, AttributeError):
         raise ValueError("Invalid descriptor type '{}.{}"
                          "".format(module, type_name))
 
@@ -101,12 +103,13 @@ class DictItemsModel(QStandardItemModel):
             self.appendRow([key_item, value_item])
 
     def get_dict(self):
-        dict = {}
-        for row in range(self.rowCount()):
-            key_item = self.item(row, 0)
-            value_item = self.item(row, 1)
-            dict[str(key_item.text())] = str(value_item.text())
-        return dict
+        # Use the same functionality that parses attributes
+        # when reading text files
+        return Orange.data.Flags([
+            "{}={}".format(self.item(row, 0).text(),
+                           self.item(row, 1).text())
+            for row in range(self.rowCount())
+        ]).attributes
 
 
 class VariableEditor(QWidget):
@@ -119,6 +122,7 @@ class VariableEditor(QWidget):
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
+        self.var = None
         self.setup_gui()
 
     def setup_gui(self):
@@ -209,11 +213,11 @@ class VariableEditor(QWidget):
     def get_data(self):
         """Retrieve the modified variable.
         """
-        name = str(self.name_edit.text())
+        name = str(self.name_edit.text()).strip()
         labels = self.labels_model.get_dict()
 
         # Is the variable actually changed.
-        if not self.is_same():
+        if self.var is not None and not self.is_same():
             var = type(self.var)(name)
             var.attributes.update(labels)
             self.var = var
@@ -222,13 +226,17 @@ class VariableEditor(QWidget):
 
         return var
 
+    def is_legal(self):
+        name = str(self.name_edit.text()).strip()
+        return not len(name) == 0
+
     def is_same(self):
         """Is the current model state the same as the input.
         """
-        name = str(self.name_edit.text())
+        name = str(self.name_edit.text()).strip()
         labels = self.labels_model.get_dict()
-
-        return self.var and name == self.var.name and labels == self.var.attributes
+        return (self.var is not None and name == self.var.name and
+                labels == self.var.attributes)
 
     def clear(self):
         """Clear the editor state.
@@ -238,7 +246,7 @@ class VariableEditor(QWidget):
         self.labels_model.set_dict({})
 
     def maybe_commit(self):
-        if not self.is_same():
+        if not self.is_same() and self.is_legal():
             self.commit()
 
     def commit(self):
@@ -312,11 +320,11 @@ class DiscreteVariableEditor(VariableEditor):
     def get_data(self):
         """Retrieve the modified variable
         """
-        name = str(self.name_edit.text())
+        name = str(self.name_edit.text()).strip()
         labels = self.labels_model.get_dict()
         values = map(str, self.values_model)
 
-        if not self.is_same():
+        if self.var is not None and not self.is_same():
             var = type(self.var)(name, values=values)
             var.attributes.update(labels)
             self.var = var
@@ -328,8 +336,9 @@ class DiscreteVariableEditor(VariableEditor):
     def is_same(self):
         """Is the current model state the same as the input.
         """
-        values = map(str, self.values_model)
-        return VariableEditor.is_same(self) and self.var.values == values
+        values = list(map(str, self.values_model))
+        return (VariableEditor.is_same(self) and self.var is not None and
+                self.var.values == values)
 
     def clear(self):
         """Clear the model state.
@@ -361,7 +370,7 @@ class OWEditDomain(widget.OWWidget):
     domain_change_hints = settings.ContextSetting({})
     selected_index = settings.ContextSetting({})
 
-    autocommit = settings.Setting(False)
+    autocommit = settings.Setting(True)
 
     def __init__(self):
         super().__init__()
@@ -388,7 +397,7 @@ class OWEditDomain(widget.OWWidget):
         gui.auto_commit(self.controlArea, self, "autocommit", "Apply")
 
         box = gui.vBox(self.mainArea, "Edit")
-        self.editor_stack = QtGui.QStackedWidget()
+        self.editor_stack = QStackedWidget()
 
         self.editor_stack.addWidget(DiscreteVariableEditor())
         self.editor_stack.addWidget(ContinuousVariableEditor())
@@ -396,6 +405,10 @@ class OWEditDomain(widget.OWWidget):
 
         box.layout().addWidget(self.editor_stack)
 
+        self.Error.add_message(
+            "duplicate_var_name",
+            "A variable name is duplicated.")
+
     @check_sql_input
     def set_data(self, data):
         """Set input data set."""
@@ -423,7 +436,7 @@ class OWEditDomain(widget.OWWidget):
         ind = self.selected_var_index()
         if ind >= 0:
             var = self.input_vars[ind]
-            desc = variable_description(var)
+            desc = variable_description(var, skip_attributes=True)
             if desc in self.domain_change_hints:
                 del self.domain_change_hints[desc]
 
@@ -454,7 +467,7 @@ class OWEditDomain(widget.OWWidget):
     def _restore(self):
         # Restore the variable states from saved settings.
         def transform(var):
-            vdesc = variable_description(var)
+            vdesc = variable_description(var, skip_attributes=True)
             if vdesc in self.domain_change_hints:
                 return variable_from_description(
                     self.domain_change_hints[vdesc],
@@ -508,13 +521,15 @@ class OWEditDomain(widget.OWWidget):
 
         # Replace the variable in the 'Domain Features' view/model
         old_var = self.input_vars[self.selected_index]
-        new_var = editor.get_data().copy(compute_value=Orange.preprocess.transformation.Identity(old_var))
+        new_var = editor.get_data().copy(
+            compute_value=Orange.preprocess.transformation.Identity(old_var)
+        )
         self.domain_model[self.selected_index] = new_var
 
 
         # Store the transformation hint.
-        self.domain_change_hints[variable_description(old_var)] = \
-                    variable_description(new_var)
+        old_var_desc = variable_description(old_var, skip_attributes=True)
+        self.domain_change_hints[old_var_desc] = variable_description(new_var)
 
         self._invalidate()
 
@@ -524,17 +539,21 @@ class OWEditDomain(widget.OWWidget):
     def commit(self):
         """Send the changed data to output."""
         new_data = None
+        var_names = [vn.name for vn in self.domain_model]
+        self.Error.duplicate_var_name.clear()
         if self.data is not None:
-            input_domain = self.data.domain
-            n_attrs = len(input_domain.attributes)
-            n_vars = len(input_domain.variables)
-            n_class_vars = len(input_domain.class_vars)
-            all_new_vars = list(self.domain_model)
-            attrs = all_new_vars[: n_attrs]
-            class_vars = all_new_vars[n_attrs: n_attrs + n_class_vars]
-            new_metas = all_new_vars[n_attrs + n_class_vars:]
-            new_domain = Orange.data.Domain(attrs, class_vars, new_metas)
-            new_data = self.data.from_table(new_domain, self.data)
+            if len(var_names) == len(set(var_names)):
+                input_domain = self.data.domain
+                n_attrs = len(input_domain.attributes)
+                n_class_vars = len(input_domain.class_vars)
+                all_new_vars = list(self.domain_model)
+                attrs = all_new_vars[: n_attrs]
+                class_vars = all_new_vars[n_attrs: n_attrs + n_class_vars]
+                new_metas = all_new_vars[n_attrs + n_class_vars:]
+                new_domain = Orange.data.Domain(attrs, class_vars, new_metas)
+                new_data = self.data.from_table(new_domain, self.data)
+            else:
+                self.Error.duplicate_var_name()
 
         self.send("Data", new_data)
 
@@ -543,9 +562,12 @@ class OWEditDomain(widget.OWWidget):
         return sh.expandedTo(QSize(660, 550))
 
     def send_report(self):
-        self.report_raw("", EditDomainReport(
-            old_domain=chain(self.data.domain.variables, self.data.domain.metas),
-            new_domain=self.domain_model).to_html())
+        if self.data is not None:
+            self.report_raw("", EditDomainReport(
+                old_domain=chain(self.data.domain.variables, self.data.domain.metas),
+                new_domain=self.domain_model).to_html())
+        else:
+            self.report_data(None)
 
 
 class EditDomainReport:
@@ -623,7 +645,7 @@ class EditDomainReport:
 
 
 def main():
-    from PyQt4.QtGui import QApplication
+    from AnyQt.QtWidgets import QApplication
     app = QApplication([])
     w = OWEditDomain()
     data = Orange.data.Table("iris")
