@@ -12,6 +12,9 @@ import functools
 import builtins
 import math
 import random
+import logging
+
+from traceback import format_exception_only
 from collections import namedtuple, OrderedDict
 from itertools import chain, count
 
@@ -335,6 +338,7 @@ class OWFeatureConstructor(OWWidget):
 
     class Error(OWWidget.Error):
         more_values_needed = Msg("Discrete feature {} needs more values.")
+        invalid_expressions = Msg("Invalid expressions: {}.")
 
     def __init__(self):
         super().__init__()
@@ -549,11 +553,7 @@ class OWFeatureConstructor(OWWidget):
                     return var.name
         return None
 
-    def apply(self):
-        if self.data is None:
-            return
-
-        desc = list(self.featuremodel)
+    def _validate_descriptors(self, desc):
 
         def validate(source):
             try:
@@ -561,11 +561,28 @@ class OWFeatureConstructor(OWWidget):
             except Exception:
                 return False
 
-        def remove_invalid_expression(desc):
-            return (desc if validate(desc.expression)
-                    else desc._replace(expression=""))
+        final = []
+        invalid = []
+        for d in desc:
+            if validate(d.expression):
+                final.append(d)
+            else:
+                final.append(d._replace(expression=""))
+                invalid.append(d)
 
-        desc = map(remove_invalid_expression, desc)
+        if invalid:
+            self.Error.invalid_expressions(", ".join(s.name for s in invalid))
+
+        return final
+
+    def apply(self):
+        self.Error.clear()
+
+        if self.data is None:
+            return
+
+        desc = list(self.featuremodel)
+        desc = self._validate_descriptors(desc)
         source_vars = tuple(self.data.domain) + self.data.domain.metas
         new_variables = construct_variables(desc, source_vars)
 
@@ -577,11 +594,12 @@ class OWFeatureConstructor(OWWidget):
             metas=self.data.domain.metas + tuple(metas)
         )
 
-        self.Error.clear()
         try:
             data = Orange.data.Table(new_domain, self.data)
         except Exception as err:
-            self.error(err.args[0])
+            log = logging.getLogger(__name__)
+            log.error("", exc_info=True)
+            self.error("".join(format_exception_only(type(err), err)).rstrip())
             return
         disc_attrs_not_ok = self.check_attrs_values(
             [var for var in attrs if var.is_discrete], data)
@@ -801,7 +819,10 @@ def construct_variables(descriptions, source_vars):
 
 
 def sanitized_name(name):
-    return re.sub(r"\W", "_", name)
+    sanitized = re.sub(r"\W", "_", name)
+    if sanitized[0].isdigit():
+        sanitized = "_" + sanitized
+    return sanitized
 
 
 def bind_variable(descriptor, env):
