@@ -5,8 +5,7 @@ from warnings import catch_warnings
 import numpy as np
 from AnyQt.QtWidgets import \
     QStyle, QComboBox, QMessageBox, QFileDialog, QGridLayout, QLabel, \
-    QLineEdit
-from AnyQt.QtWidgets import QSizePolicy as Policy
+    QLineEdit, QSizePolicy as Policy
 from AnyQt.QtCore import Qt, QTimer, QSize
 
 from Orange.canvas.gui.utils import OSX_NSURL_toLocalFile
@@ -87,7 +86,9 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
     SIZE_LIMIT = 1e7
     LOCAL_FILE, URL = range(2)
 
-    settingsHandler = PerfectDomainContextHandler()
+    settingsHandler = PerfectDomainContextHandler(
+        match_values=PerfectDomainContextHandler.MATCH_VALUES_ALL
+    )
 
     # Overload RecentPathsWidgetMixin.recent_paths to set defaults
     recent_paths = Setting([
@@ -117,6 +118,9 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         file_too_big = widget.Msg("The file is too large to load automatically."
                                   " Press Reload to load.")
 
+    class Error(widget.OWWidget.Error):
+        file_not_found = widget.Msg("File not found.")
+
     def __init__(self):
         super().__init__()
         RecentPathsWComboMixin.__init__(self)
@@ -219,7 +223,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
         if self.source == self.LOCAL_FILE:
             last_path = self.last_path()
-            if last_path and os.path.getsize(last_path) > self.SIZE_LIMIT:
+            if last_path and os.path.exists(last_path) and \
+                    os.path.getsize(last_path) > self.SIZE_LIMIT:
                 self.Warning.file_too_big()
                 return
 
@@ -259,7 +264,6 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
             self, 'Open Orange Data File', start_file, self.dlg_formats)
         if not filename:
             return
-        self.loaded_file = filename
         self.add_path(filename)
         self.source = self.LOCAL_FILE
         self.load_data()
@@ -272,7 +276,13 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
         self.closeContext()
         self.domain_editor.set_domain(None)
         self.apply_button.setEnabled(False)
-        self.Warning.file_too_big.clear()
+        self.clear_messages()
+        self.set_file_list()
+        if self.last_path() and not os.path.exists(self.last_path()):
+            self.Error.file_not_found()
+            self.send("Data", None)
+            self.info.setText("No data.")
+            return
 
         error = None
         try:
@@ -305,7 +315,8 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
 
         self.info.setText(self._describe(data))
 
-        add_origin(data, self.loaded_file or self.last_path())
+        self.loaded_file = self.last_path()
+        add_origin(data, self.loaded_file)
         self.data = data
         self.openContext(data.domain)
         self.apply_domain_edit()  # sends data
@@ -426,7 +437,7 @@ class OWFile(widget.OWWidget, RecentPathsWComboMixin):
             home = os.path.expanduser("~")
             if self.loaded_file.startswith(home):
                 # os.path.join does not like ~
-                name = "~/" + \
+                name = "~" + os.path.sep + \
                        self.loaded_file[len(home):].lstrip("/").lstrip("\\")
             else:
                 name = self.loaded_file
