@@ -3,42 +3,42 @@ import sys
 
 from collections import namedtuple, OrderedDict
 from itertools import chain
-from functools import reduce
 from contextlib import contextmanager
 
 import numpy
-import scipy.cluster.hierarchy
 
-from PyQt4.QtGui import (
+from AnyQt.QtWidgets import (
     QGraphicsWidget, QGraphicsObject, QGraphicsLinearLayout, QGraphicsPathItem,
-    QGraphicsScene, QGraphicsView, QTransform, QPainterPath,
-    QColor, QBrush, QPen, QFontMetrics, QGridLayout, QFormLayout,
-    QSizePolicy, QGraphicsSimpleTextItem, QPolygonF, QPainterPathStroker,
-    QGraphicsLayoutItem, QAction, QKeySequence, QFont
+    QGraphicsScene, QGraphicsView, QGridLayout, QFormLayout, QSizePolicy,
+    QGraphicsSimpleTextItem,
+    QGraphicsLayoutItem, QAction,
 )
-
-from PyQt4.QtCore import Qt,  QSize, QSizeF, QPointF, QRectF, QLineF, QEvent
-from PyQt4.QtCore import pyqtSignal as Signal
+from AnyQt.QtGui import (
+    QTransform, QPainterPath, QPainterPathStroker, QColor, QBrush, QPen,
+    QFont, QFontMetrics, QPolygonF, QKeySequence
+)
+from AnyQt.QtCore import Qt, QSize, QSizeF, QPointF, QRectF, QLineF, QEvent
+from AnyQt.QtCore import pyqtSignal as Signal
 
 import pyqtgraph as pg
 
 import Orange.data
 from Orange.data.domain import filter_visible
+from Orange.data import Domain
 import Orange.misc
 from Orange.clustering.hierarchical import \
-    postorder, preorder, Tree, tree_from_linkage, leaves, prune, top_clusters
+    postorder, preorder, Tree, tree_from_linkage, dist_matrix_linkage, \
+    leaves, prune, top_clusters
 
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import colorpalette, itemmodels
-from Orange.widgets.io import FileFormat
+from Orange.widgets.utils.annotated_data import (create_annotated_table,
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 
 __all__ = ["OWHierarchicalClustering"]
 
-# In scipy 0.14 ward linkage cannot be computed from the distance
-# matrix alone, it requires the full data matrix (in 0.15 the whole
-# hierarchical clustering is/will be reimplemented and from the
-# looks of it will support ward from dist matrix).
-LINKAGE = ["Single", "Average", "Weighted", "Complete"]
+
+LINKAGE = ["Single", "Average", "Weighted", "Complete", "Ward"]
 
 
 def dendrogram_layout(tree, expand_leaves=False):
@@ -74,7 +74,7 @@ Point = namedtuple("Point", ["x", "y"])
 Element = namedtuple("Element", ["anchor", "path"])
 
 
-def Path_toQtPath(geom):
+def path_toQtPath(geom):
     p = QPainterPath()
     anchor, points = geom
     if len(points) > 1:
@@ -578,7 +578,7 @@ class DendrogramWidget(QGraphicsWidget):
             item = self._items[node]
             item.element = geom
 
-            item.setPath(Path_toQtPath(geom))
+            item.setPath(path_toQtPath(geom))
             item.setZValue(-node.value.height)
             r = item.path().boundingRect()
             base = self._root.value.height
@@ -604,10 +604,20 @@ class DendrogramWidget(QGraphicsWidget):
         else:
             drect = QSizeF(leaf_count, self._root.value.height)
 
-        transform = QTransform().scale(
-            crect.width() / drect.width(),
-            crect.height() / drect.height()
-        )
+        eps = numpy.finfo(numpy.float64).eps
+
+        if abs(drect.width()) < eps:
+            sx = 1.0
+        else:
+            sx = crect.width() / drect.width()
+
+        if abs(drect.height()) < eps:
+            sy = 1.0
+        else:
+            sy = crect.height() / drect.height()
+
+        transform = QTransform().scale(sx, sy)
+
         self._itemgroup.setPos(crect.topLeft())
         self._itemgroup.setTransform(transform)
         self._selection_items = None
@@ -710,12 +720,18 @@ class OWHierarchicalClustering(widget.OWWidget):
     inputs = [("Distances", Orange.misc.DistMatrix, "set_distances")]
 
     outputs = [("Selected Data", Orange.data.Table, widget.Default),
-               ("Other Data", Orange.data.Table)]
+               (ANNOTATED_DATA_SIGNAL_NAME, Orange.data.Table)]
+
+    settingsHandler = settings.DomainContextHandler()
 
     #: Selected linkage
     linkage = settings.Setting(1)
     #: Index of the selected annotation item (variable, ...)
-    annotation_idx = settings.Setting(0)
+    annotation = settings.ContextSetting("Enumeration", exclude_metas=False)
+    #: Out-of-context setting for the case when the "Name" option is available
+    annotation_if_names = settings.Setting("Name")
+    #: Out-of-context setting for the case with just "Enumerate" and "None"
+    annotation_if_enumerate = settings.Setting("Enumerate")
     #: Selected tree pruning (none/max depth)
     pruning = settings.Setting(0)
     #: Maximum depth when max depth pruning is selected
@@ -742,6 +758,7 @@ class OWHierarchicalClustering(widget.OWWidget):
     AttributeRole, ClassRole, MetaRole = 0, 1, 2
 
     cluster_roles = ["Attribute", "Class variable", "Meta variable"]
+    basic_annotations = ["None", "Enumeration"]
 
     def __init__(self):
         super().__init__()
@@ -757,11 +774,11 @@ class OWHierarchicalClustering(widget.OWWidget):
             self.controlArea, self, "linkage", items=LINKAGE, box="Linkage",
             callback=self._invalidate_clustering)
 
+        model = itemmodels.VariableListModel()
+        model[:] = self.basic_annotations
         self.label_cb = gui.comboBox(
-            self.controlArea, self, "annotation_idx", box="Annotation",
-            callback=self._update_labels, contentsLength=12)
-        self.label_cb.setModel(itemmodels.VariableListModel())
-        self.label_cb.model()[:] = ["None", "Enumeration"]
+            self.controlArea, self, "annotation", box="Annotation",
+            model=model, callback=self._update_labels, contentsLength=12)
 
         box = gui.radioButtons(
             self.controlArea, self, "pruning", box="Pruning",
@@ -783,37 +800,39 @@ class OWHierarchicalClustering(widget.OWWidget):
             1, 0)
         grid.addWidget(self.max_depth_spin, 1, 1)
 
-        box = gui.radioButtons(
+        self.selection_box = gui.radioButtons(
             self.controlArea, self, "selection_method",
             box="Selection",
             callback=self._selection_method_changed)
 
         grid = QGridLayout()
-        box.layout().addLayout(grid)
+        self.selection_box.layout().addLayout(grid)
         grid.addWidget(
-            gui.appendRadioButton(box, "Manual", addToLayout=False),
+            gui.appendRadioButton(
+                self.selection_box, "Manual", addToLayout=False),
             0, 0
         )
         grid.addWidget(
-            gui.appendRadioButton(box, "Height ratio:", addToLayout=False),
+            gui.appendRadioButton(
+                self.selection_box, "Height ratio:", addToLayout=False),
             1, 0
         )
         self.cut_ratio_spin = gui.spin(
-            box, self, "cut_ratio", 0, 100, step=1e-1, spinType=float,
-            callback=self._selection_method_changed
+            self.selection_box, self, "cut_ratio", 0, 100, step=1e-1,
+            spinType=float, callback=self._selection_method_changed
         )
         self.cut_ratio_spin.setSuffix("%")
 
         grid.addWidget(self.cut_ratio_spin, 1, 1)
 
         grid.addWidget(
-            gui.appendRadioButton(box, "Top N:", addToLayout=False),
+            gui.appendRadioButton(
+                self.selection_box, "Top N:", addToLayout=False),
             2, 0
         )
-        self.top_n_spin = gui.spin(box, self, "top_n", 1, 20,
+        self.top_n_spin = gui.spin(self.selection_box, self, "top_n", 1, 20,
                                    callback=self._selection_method_changed)
         grid.addWidget(self.top_n_spin, 2, 1)
-        box.layout().addLayout(grid)
 
         self.zoom_slider = gui.hSlider(
             self.controlArea, self, "zoom_factor", box="Zoom",
@@ -939,7 +958,6 @@ class OWHierarchicalClustering(widget.OWWidget):
 
     def set_distances(self, matrix):
         self.error()
-        self._set_items(None)
         if matrix is not None:
             N, _ = matrix.shape
             if N < 2:
@@ -947,38 +965,46 @@ class OWHierarchicalClustering(widget.OWWidget):
                 matrix = None
 
         self.matrix = matrix
-        self._invalidate_clustering()
-
         if matrix is not None:
             self._set_items(matrix.row_items, matrix.axis)
+        else:
+            self._set_items(None)
+        self._invalidate_clustering()
 
         self.unconditional_commit()
 
     def _set_items(self, items, axis=1):
+        self.closeContext()
         self.items = items
         model = self.label_cb.model()
-        if items is None:
-            model[:] = ["None", "Enumeration"]
-        elif not axis:
-            model[:] = ["None", "Enumeration", "Attribute names"]
-            self.annotation_idx = 2
-        elif isinstance(items, Orange.data.Table):
+        if len(model) == 3:
+            self.annotation_if_names = self.annotation
+        elif len(model) == 2:
+            self.annotation_if_enumerate = self.annotation
+        if isinstance(items, Orange.data.Table) and axis:
             model[:] = chain(
-                ["None", "Enumeration"],
+                self.basic_annotations,
                 [model.Separator],
                 items.domain.class_vars,
                 items.domain.metas,
                 [model.Separator] if (items.domain.class_vars or items.domain.metas) and
-                                     next(filter_visible(items.domain.attributes), False) else [],
+                next(filter_visible(items.domain.attributes), False) else [],
                 filter_visible(items.domain.attributes)
             )
-        elif isinstance(items, list) and \
-                all(isinstance(var, Orange.data.Variable) for var in items):
-            model[:] = ["None", "Enumeration", "Name"]
+            if items.domain.class_vars:
+                self.annotation = items.domain.class_vars[0]
+            else:
+                self.annotation = "Enumeration"
+            self.openContext(items.domain)
         else:
-            model[:] = ["None", "Enumeration"]
-        self.annotation_idx = min(self.annotation_idx,
-                                  len(model) - 1)
+            name_option = bool(
+                items is not None and (
+                    not axis or
+                    isinstance(items, list) and
+                    all(isinstance(var, Orange.data.Variable) for var in items)))
+            model[:] = self.basic_annotations + ["Name"] * name_option
+            self.annotation = self.annotation_if_names if name_option \
+                else self.annotation_if_enumerate
 
     def _clear_plot(self):
         self.labels.set_labels([])
@@ -1003,14 +1029,9 @@ class OWHierarchicalClustering(widget.OWWidget):
         distances = self.matrix
 
         if distances is not None:
-            # Convert to flat upper triangular distances
-            i, j = numpy.triu_indices(distances.shape[0], k=1)
-            distances = numpy.asarray(distances[i, j])
-
             method = LINKAGE[self.linkage].lower()
-            Z = scipy.cluster.hierarchy.linkage(
-                distances, method=method
-            )
+            Z = dist_matrix_linkage(distances, linkage=method)
+
             tree = tree_from_linkage(Z)
             self.linkmatrix = Z
             self.root = tree
@@ -1034,17 +1055,16 @@ class OWHierarchicalClustering(widget.OWWidget):
         if self.root and self._displayed_root:
             indices = [leaf.value.index for leaf in leaves(self.root)]
 
-            if self.annotation_idx == 0:
+            if self.annotation == "None":
                 labels = []
-            elif self.annotation_idx == 1:
+            elif self.annotation == "Enumeration":
                 labels = [str(i+1) for i in indices]
-            elif self.label_cb.model()[self.annotation_idx] == "Attribute names":
+            elif self.annotation == "Name":
                 attr = self.matrix.row_items.domain.attributes
                 labels = [str(attr[i]) for i in indices]
-            elif isinstance(self.items, Orange.data.Table):
-                var = self.label_cb.model()[self.annotation_idx]
-                col_data, _ = self.items.get_column_view(var)
-                labels = [var.str_val(val) for val in col_data]
+            elif isinstance(self.annotation, Orange.data.Variable):
+                col_data, _ = self.items.get_column_view(self.annotation)
+                labels = [self.annotation.str_val(val) for val in col_data]
                 labels = [labels[idx] for idx in indices]
             else:
                 labels = []
@@ -1060,6 +1080,7 @@ class OWHierarchicalClustering(widget.OWWidget):
     def _invalidate_clustering(self):
         self._update()
         self._update_labels()
+        self._invalidate_output()
 
     def _invalidate_output(self):
         self.commit()
@@ -1083,7 +1104,8 @@ class OWHierarchicalClustering(widget.OWWidget):
     def commit(self):
         items = getattr(self.matrix, "items", self.items)
         if not items:
-            # nothing to commit
+            self.send("Selected Data", None)
+            self.send(ANNOTATED_DATA_SIGNAL_NAME, None)
             return
 
         selection = self.dendrogram.selected_nodes()
@@ -1100,10 +1122,12 @@ class OWHierarchicalClustering(widget.OWWidget):
 
         if not selected_indices:
             self.send("Selected Data", None)
-            self.send("Other Data", None)
+            annotated_data = create_annotated_table(items, []) \
+                if self.selection_method == 0 and self.matrix.axis else None
+            self.send(ANNOTATED_DATA_SIGNAL_NAME, annotated_data)
             return
 
-        selected_data = unselected_data = None
+        selected_data = None
 
         if isinstance(items, Orange.data.Table) and self.matrix.axis == 1:
             # Select rows
@@ -1120,7 +1144,7 @@ class OWHierarchicalClustering(widget.OWWidget):
                     str(self.cluster_name),
                     values=["C{}".format(i + 1)
                             for i in range(len(maps))] +
-                           ["Other"]
+                    ["Other"]
                 )
                 data, domain = items, items.domain
 
@@ -1143,8 +1167,20 @@ class OWHierarchicalClustering(widget.OWWidget):
 
             if selected_indices:
                 selected_data = data[mask]
-            if unselected_indices:
-                unselected_data = data[~mask]
+                if self.append_clusters:
+                    def remove_other_value(vars_):
+                        vars_ = list(vars_)
+                        clust_var = vars_[-1].copy()
+                        clust_var.values.pop()
+                        vars_[-1] = clust_var
+                        return vars_
+                    if self.cluster_role == self.AttributeRole:
+                        attrs = remove_other_value(attrs)
+                    elif self.cluster_role == self.ClassRole:
+                        class_ = remove_other_value(class_)
+                    elif self.cluster_role == self.MetaRole:
+                        metas = remove_other_value(metas)
+                    selected_data.domain = Domain(attrs, class_, metas)
 
         elif isinstance(items, Orange.data.Table) and self.matrix.axis == 0:
             # Select columns
@@ -1152,13 +1188,12 @@ class OWHierarchicalClustering(widget.OWWidget):
                 [items.domain[i] for i in selected_indices],
                 items.domain.class_vars, items.domain.metas)
             selected_data = items.from_table(domain, items)
-            domain = Orange.data.Domain(
-                [items.domain[i] for i in unselected_indices],
-                items.domain.class_vars, items.domain.metas)
-            unselected_data = items.from_table(domain, items)
+            data = None
 
         self.send("Selected Data", selected_data)
-        self.send("Other Data", unselected_data)
+        annotated_data = create_annotated_table(data, selected_indices) if \
+            self.selection_method == 0 else None
+        self.send(ANNOTATED_DATA_SIGNAL_NAME, annotated_data)
 
     def sizeHint(self):
         return QSize(800, 500)
@@ -1288,6 +1323,7 @@ class OWHierarchicalClustering(widget.OWWidget):
         # dendrogram view.
         self.selection_method = 0
         self._selection_method_changed()
+        self._invalidate_output()
 
     def __zoom_in(self):
         def clip(minval, maxval, val):
@@ -1326,7 +1362,7 @@ class OWHierarchicalClustering(widget.OWWidget):
 
     def send_report(self):
         annot = self.label_cb.currentText()
-        if self.annotation_idx <= 1:
+        if isinstance(self.annotation, str):
             annot = annot.lower()
         if self.selection_method == 0:
             sel = "manual"
@@ -1612,7 +1648,7 @@ def clusters_at_height(root, height):
 
 
 def main(argv=None):
-    from PyQt4.QtGui import QApplication
+    from AnyQt.QtWidgets import QApplication
     import sip
     import Orange.distance as distance
 
