@@ -1,3 +1,5 @@
+import numbers
+
 from AnyQt.QtWidgets import QFormLayout, QLineEdit
 from AnyQt.QtGui import QColor
 from AnyQt.QtCore import Qt, QTimer
@@ -18,6 +20,10 @@ except ImportError:
     remotely = False
 
 
+# Maximum number of PCA components that we can set in the widget
+MAX_COMPONENTS = 100
+
+
 class OWPCA(widget.OWWidget):
     name = "PCA"
     description = "Principal component analysis with a scree-diagram."
@@ -41,6 +47,15 @@ class OWPCA(widget.OWWidget):
 
     graph_name = "plot.plotItem"
 
+    class Warning(widget.OWWidget.Warning):
+        trivial_components = widget.Msg(
+            "All components of the PCA are trivial (explain 0 variance). "
+            "Input data is constant (or near constant).")
+
+    class Error(widget.OWWidget.Error):
+        no_features = widget.Msg("At least 1 feature is required")
+        no_instances = widget.Msg("At least 1 data instance is required")
+
     def __init__(self):
         super().__init__()
         self.data = None
@@ -50,7 +65,8 @@ class OWPCA(widget.OWWidget):
         self._variance_ratio = None
         self._cumulative = None
         self._line = False
-        self._pca_projector = PCA()
+        # max_components limit allows scikit-learn to select a faster method for big data
+        self._pca_projector = PCA(max_components=MAX_COMPONENTS)
         self._pca_projector.component = self.ncomponents
         self._pca_preprocessors = PCA.preprocessors
 
@@ -60,7 +76,7 @@ class OWPCA(widget.OWWidget):
         box.layout().addLayout(form)
 
         self.components_spin = gui.spin(
-            box, self, "ncomponents", 0, 1000,
+            box, self, "ncomponents", 1, MAX_COMPONENTS,
             callback=self._update_selection_component_spin,
             keyboardTracking=False
         )
@@ -109,7 +125,7 @@ class OWPCA(widget.OWWidget):
         gui.checkBox(self.options_box, self, "normalize", "Normalize data",
                      callback=self._update_normalize)
         self.maxp_spin = gui.spin(
-            self.options_box, self, "maxp", 1, 100,
+            self.options_box, self, "maxp", 1, MAX_COMPONENTS,
             label="Show only first", callback=self._setup_plot,
             keyboardTracking=False
         )
@@ -171,6 +187,7 @@ class OWPCA(widget.OWWidget):
         self.fit()
 
     def fit(self):
+        self.clear_messages()
         self.clear()
         self.start_button.setEnabled(False)
         if self.data is None:
@@ -183,15 +200,31 @@ class OWPCA(widget.OWWidget):
             self.start_button.setEnabled(True)
         else:
             self.sampling_box.setVisible(False)
+            if len(data.domain.attributes) == 0:
+                self.Error.no_features()
+                self.send("Transformed data", None)
+                self.send("Components", None)
+                self.send("PCA", None)
+                return
+            if len(data) == 0:
+                self.Error.no_instances()
+                self.send("Transformed data", None)
+                self.send("Components", None)
+                self.send("PCA", None)
+                return
+
             pca = self._pca_projector(data)
             variance_ratio = pca.explained_variance_ratio_
             cumulative = numpy.cumsum(variance_ratio)
-            self.components_spin.setRange(0, len(cumulative))
 
-            self._pca = pca
-            self._variance_ratio = variance_ratio
-            self._cumulative = cumulative
-            self._setup_plot()
+            if numpy.isfinite(cumulative[-1]):
+                self.components_spin.setRange(0, len(cumulative))
+                self._pca = pca
+                self._variance_ratio = variance_ratio
+                self._cumulative = cumulative
+                self._setup_plot()
+            else:
+                self.Warning.trivial_components()
 
             self.unconditional_commit()
 
@@ -204,6 +237,7 @@ class OWPCA(widget.OWWidget):
         self.plot_horlabels = []
         self.plot_horlines = []
         self.plot.clear()
+        self.Warning.trivial_components.clear()
 
     def get_model(self):
         if self.rpca is None:
@@ -222,6 +256,9 @@ class OWPCA(widget.OWWidget):
 
     def _setup_plot(self):
         self.plot.clear()
+        if self._pca is None:
+            return
+
         explained_ratio = self._variance_ratio
         explained = self._cumulative
         p = min(len(self._variance_ratio), self.maxp)
@@ -279,7 +316,9 @@ class OWPCA(widget.OWWidget):
         self._set_horline_pos()
 
         if self._pca is not None:
-            self.variance_covered = self._cumulative[components - 1] * 100
+            var = self._cumulative[components - 1]
+            if numpy.isfinite(var):
+                self.variance_covered = int(var * 100)
 
         if current != self._nselected_components():
             self._invalidate_selection()
@@ -295,7 +334,10 @@ class OWPCA(widget.OWWidget):
             cut = len(self._variance_ratio)
         else:
             cut = self.ncomponents
-        self.variance_covered = self._cumulative[cut - 1] * 100
+
+        var = self._cumulative[cut - 1]
+        if numpy.isfinite(var):
+            self.variance_covered = int(var * 100)
 
         if numpy.floor(self._line.value()) + 1 != cut:
             self._line.setValue(cut - 1)
@@ -339,7 +381,8 @@ class OWPCA(widget.OWWidget):
         var_max = self._cumulative[max_comp - 1]
         if var_max != numpy.floor(self.variance_covered / 100.0):
             cut = max_comp
-            self.variance_covered = var_max * 100
+            assert numpy.isfinite(var_max)
+            self.variance_covered = int(var_max * 100)
         else:
             self.ncomponents = cut = numpy.searchsorted(
                 self._cumulative, self.variance_covered / 100.0) + 1
@@ -358,7 +401,7 @@ class OWPCA(widget.OWWidget):
         transformed = components = None
         if self._pca is not None:
             if self._transformed is None:
-                # Compute the full transform (all components) only once.
+                # Compute the full transform (MAX_COMPONENTS components) only once.
                 self._transformed = self._pca(self.data)
             transformed = self._transformed
 
@@ -391,6 +434,22 @@ class OWPCA(widget.OWWidget):
         ))
         self.report_plot()
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        if "variance_covered" in settings:
+            # Due to the error in gh-1896 the variance_covered was persisted
+            # as a NaN value, causing a TypeError in the widgets `__init__`.
+            vc = settings["variance_covered"]
+            if isinstance(vc, numbers.Real):
+                if numpy.isfinite(vc):
+                    vc = int(vc)
+                else:
+                    vc = 100
+                settings["variance_covered"] = vc
+        if settings["ncomponents"] > MAX_COMPONENTS:
+            settings["ncomponents"] = MAX_COMPONENTS
+
+
 def main():
     import gc
     from AnyQt.QtWidgets import QApplication
