@@ -43,7 +43,7 @@ def font_resize(font, factor, minsize=None, maxsize=None):
 
 class ScatterPlotVizRank(VizRankDialogAttrPair):
     captionTitle = "Score Plots"
-    K = 10
+    minK = 10
 
     def check_preconditions(self):
         self.Information.add_message(
@@ -69,9 +69,9 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
         valid = graph.get_valid_list(ind12)
         X = graph.jittered_data[ind12, :][:, valid].T
         Y = self.master.data.Y[valid]
-        if X.shape[0] < self.K:
+        if X.shape[0] < self.minK:
             return
-        n_neighbors = min(self.K, len(X) - 1)
+        n_neighbors = min(self.minK, len(X) - 1)
         knn = NearestNeighbors(n_neighbors=n_neighbors).fit(X)
         ind = knn.kneighbors(return_distance=False)
         if self.master.data.domain.has_discrete_class:
@@ -92,7 +92,7 @@ class ScatterPlotVizRank(VizRankDialogAttrPair):
         data = Table(dom, X, Y)
         relief = ReliefF if isinstance(dom.class_var, DiscreteVariable) \
             else RReliefF
-        weights = relief(n_iterations=100, k_nearest=self.K)(data)
+        weights = relief(n_iterations=100, k_nearest=self.minK)(data)
         attrs = sorted(zip(weights, mdomain.attributes),
                        key=lambda x: (-x[0], x[1].name))
         return [a for _, a in attrs]
@@ -124,6 +124,7 @@ class OWScatterPlot(OWWidget):
 
     attr_x = ContextSetting(None)
     attr_y = ContextSetting(None)
+    selection = Setting(None, schema_only=True)
 
     graph = SettingProvider(OWScatterPlotGraph)
 
@@ -191,36 +192,10 @@ class OWScatterPlot(OWWidget):
             callback=self.switch_sampling, commit=lambda: self.add_data(1))
         self.sampling.setVisible(False)
 
-        box = gui.vBox(self.controlArea, "Points")
-        self.color_model = DomainModel(
-            placeholder="(Same color)", valid_types=dmod.PRIMITIVE)
-        self.cb_attr_color = gui.comboBox(
-            box, self, "graph.attr_color", label="Color:",
-            callback=self.update_colors,
-            model=self.color_model, **common_options)
-        self.label_model = DomainModel(
-            placeholder="(No labels)")
-        self.cb_attr_label = gui.comboBox(
-            box, self, "graph.attr_label", label="Label:",
-            callback=self.graph.update_labels,
-            model=self.label_model, **common_options)
-        self.shape_model = DomainModel(
-            placeholder="(Same shape)", valid_types=DiscreteVariable)
-        self.cb_attr_shape = gui.comboBox(
-            box, self, "graph.attr_shape", label="Shape:",
-            callback=self.graph.update_shapes,
-            model=self.shape_model, **common_options)
-        self.size_model = DomainModel(
-            placeholder="(Same size)", valid_types=ContinuousVariable)
-        self.cb_attr_size = gui.comboBox(
-            box, self, "graph.attr_size", label="Size:",
-            callback=self.graph.update_sizes,
-            model=self.size_model, **common_options)
-        self.models = [self.xy_model, self.color_model, self.label_model,
-                       self.shape_model, self.size_model]
-
         g = self.graph.gui
-        g.point_properties_box(self.controlArea, box)
+        g.point_properties_box(self.controlArea)
+        self.models = [self.xy_model] + g.points_models
+
         box = gui.vBox(self.controlArea, "Plot Properties")
         g.add_widgets([g.ShowLegend, g.ShowGridLines], box)
         gui.checkBox(
@@ -305,8 +280,9 @@ class OWScatterPlot(OWWidget):
     #         self.graph.selectionCurveList.append(c)
 
     def reset_graph_data(self, *_):
-        self.graph.rescale_data()
-        self.update_graph()
+        if self.data is not None:
+            self.graph.rescale_data()
+            self.update_graph()
 
     def set_data(self, data):
         self.clear_messages()
@@ -358,8 +334,8 @@ class OWScatterPlot(OWWidget):
             for el in iterable:
                 if isinstance(el, Orange.data.Variable) and el.name == name:
                     return el
-            else:
-                return None
+            return None
+
         # handle restored settings from  < 3.3.9 when attr_* were stored
         # by name
         if isinstance(self.attr_x, str):
@@ -368,16 +344,16 @@ class OWScatterPlot(OWWidget):
             self.attr_y = findvar(self.attr_y, self.xy_model)
         if isinstance(self.graph.attr_label, str):
             self.graph.attr_label = findvar(
-                self.graph.attr_label, self.label_model)
+                self.graph.attr_label, self.graph.gui.label_model)
         if isinstance(self.graph.attr_color, str):
             self.graph.attr_color = findvar(
-                self.graph.attr_color, self.color_model)
+                self.graph.attr_color, self.graph.gui.color_model)
         if isinstance(self.graph.attr_shape, str):
             self.graph.attr_shape = findvar(
-                self.graph.attr_shape, self.shape_model)
+                self.graph.attr_shape, self.graph.gui.shape_model)
         if isinstance(self.graph.attr_size, str):
             self.graph.attr_size = findvar(
-                self.graph.attr_size, self.size_model)
+                self.graph.attr_size, self.graph.gui.size_model)
 
     def add_data(self, time=0.4):
         if self.data and len(self.data) > 2000:
@@ -428,8 +404,17 @@ class OWScatterPlot(OWWidget):
         self.update_graph()
         self.cb_class_density.setEnabled(self.graph.can_draw_density())
         self.cb_reg_line.setEnabled(self.graph.can_draw_regresssion_line())
+        self.apply_selection()
         self.unconditional_commit()
 
+    def apply_selection(self):
+        """Apply selection saved in workflow."""
+        if self.data is not None and self.selection is not None:
+            self.graph.selection = np.zeros(len(self.data), dtype=np.uint8)
+            self.selection = [x for x in self.selection if x < len(self.data)]
+            self.graph.selection[self.selection] = 1
+            self.graph.update_colors(keep_colors=True)
+
     def set_shown_attributes(self, attributes):
         if attributes and len(attributes) >= 2:
             self.attribute_selection_list = attributes[:2]
@@ -462,7 +447,6 @@ class OWScatterPlot(OWWidget):
         self.send_features()
 
     def update_colors(self):
-        self.graph.update_colors()
         self.cb_class_density.setEnabled(self.graph.can_draw_density())
 
     def update_density(self):
@@ -472,6 +456,8 @@ class OWScatterPlot(OWWidget):
         self.update_graph(reset_view=False)
 
     def update_graph(self, reset_view=True, **_):
+        axis = self.graph.plot_widget.getAxis("left")
+        axis.textWidth = 0
         self.graph.zoomStack = []
         if self.graph.data is None:
             return
@@ -518,7 +504,9 @@ class OWScatterPlot(OWWidget):
         self.send("Selected Data", selected)
         self.send(ANNOTATED_DATA_SIGNAL_NAME, annotated)
 
-
+        # Store current selection in a setting that is stored in workflow
+        if self.selection is not None and len(selection):
+            self.selection = list(selection)
 
     def send_features(self):
         features = None
@@ -554,13 +542,19 @@ class OWScatterPlot(OWWidget):
         if caption:
             self.report_caption(caption)
 
+    def closeContext(self):
+        if self.current_context is not None:
+            # When dataset changes, forget selection
+            self.selection = None
+        super().closeContext()
+
     def onDeleteWidget(self):
         super().onDeleteWidget()
         self.graph.plot_widget.getViewBox().deleteLater()
         self.graph.plot_widget.clear()
 
 
-def test_main(argv=None):
+def main(argv=None):
     import sys
     if argv is None:
         argv = sys.argv
@@ -590,4 +584,4 @@ def test_main(argv=None):
     return rval
 
 if __name__ == "__main__":
-    test_main()
+    main()
