@@ -1,29 +1,34 @@
+import sys
 import itertools
 from xml.sax.saxutils import escape
 from math import log10, floor, ceil
 
 import numpy as np
+from scipy.stats import linregress
+
+from AnyQt.QtCore import Qt, QObject, QEvent, QRectF, QPointF, QSize
+from AnyQt.QtGui import (
+    QStaticText, QColor, QPen, QBrush, QPainterPath, QTransform, QPainter)
+from AnyQt.QtWidgets import QApplication, QToolTip, QPinchGesture, \
+    QGraphicsTextItem, QGraphicsRectItem
+
 import pyqtgraph as pg
 from pyqtgraph.graphicsItems.ViewBox import ViewBox
 import pyqtgraph.graphicsItems.ScatterPlotItem
-from pyqtgraph.graphicsItems.ImageItem import ImageItem
 from pyqtgraph.graphicsItems.LegendItem import LegendItem, ItemSample
 from pyqtgraph.graphicsItems.ScatterPlotItem import ScatterPlotItem
 from pyqtgraph.graphicsItems.TextItem import TextItem
+from pyqtgraph.graphicsItems.InfiniteLine import InfiniteLine
 from pyqtgraph.Point import Point
-from PyQt4.QtCore import Qt, QObject, QEvent, QRectF, QPointF
-from PyQt4 import QtCore
-from PyQt4.QtGui import QApplication, QColor, QPen, QBrush, QToolTip
-from PyQt4.QtGui import QStaticText, QPainterPath, QTransform, QPinchGesture, QPainter
+
 
 from Orange.widgets import gui
-from Orange.widgets.utils import classdensity
+from Orange.widgets.utils import classdensity, get_variable_values_sorted
 from Orange.widgets.utils.colorpalette import (ColorPaletteGenerator,
                                                ContinuousPaletteGenerator)
 from Orange.widgets.utils.plot import \
     OWPalette, OWPlotGUI, SELECT, PANNING, ZOOMING
-from Orange.widgets.utils.scaling import (get_variable_values_sorted,
-                                          ScaleScatterPlotData)
+from Orange.widgets.utils.scaling import ScaleScatterPlotData
 from Orange.widgets.settings import Setting, ContextSetting
 
 
@@ -296,7 +301,11 @@ class DiscretizedScale:
         """
         super().__init__()
         dif = max_v - min_v if max_v != min_v else 1
-        decimals = -floor(log10(dif))
+        if np.isnan(dif):
+            min_v = 0
+            dif = decimals = 1
+        else:
+            decimals = -floor(log10(dif))
         resolution = 10 ** -decimals
         bins = ceil(dif / resolution)
         if bins < 6:
@@ -349,12 +358,17 @@ class InteractiveViewBox(ViewBox):
             pos = ev.pos()
             if ev.button() == Qt.LeftButton:
                 self.safe_update_scale_box(ev.buttonDownPos(), ev.pos())
+                scene = self.scene()
+                dragtip = scene.drag_tooltip
                 if ev.isFinish():
+                    dragtip.hide()
                     self.rbScaleBox.hide()
                     pixel_rect = QRectF(ev.buttonDownPos(ev.button()), pos)
                     value_rect = self.childGroup.mapRectFromParent(pixel_rect)
                     self.graph.select_by_rectangle(value_rect)
                 else:
+                    dragtip.setPos(10, self.height() + 3)
+                    dragtip.show()  # although possibly already shown
                     self.safe_update_scale_box(ev.buttonDownPos(), ev.pos())
         elif self.graph.state == ZOOMING or self.graph.state == PANNING:
             ev.ignore()
@@ -396,7 +410,7 @@ class InteractiveViewBox(ViewBox):
         super().scaleHistory(d)
 
     def mouseClickEvent(self, ev):
-        if ev.button() == QtCore.Qt.RightButton:  # undo zoom
+        if ev.button() == Qt.RightButton:  # undo zoom
             self.scaleHistory(-1)
         else:
             ev.accept()
@@ -453,11 +467,15 @@ _define_symbols()
 
 
 class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
-    attr_color = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
-    attr_label = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_color = ContextSetting(
+        None, ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_label = ContextSetting(
+        None, ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_shape = ContextSetting(
+        None, ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_size = ContextSetting(
+        None, ContextSetting.OPTIONAL, exclude_metas=False)
     label_only_selected = Setting(False)
-    attr_shape = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
-    attr_size = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
 
     point_width = Setting(10)
     alpha_value = Setting(128)
@@ -465,6 +483,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
     show_legend = Setting(True)
     tooltip_shows_all = Setting(False)
     class_density = Setting(False)
+    show_reg_line = Setting(False)
     resolution = 256
 
     CurveSymbols = np.array("o x t + d s ?".split())
@@ -479,13 +498,17 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                                          background="w")
         self.plot_widget.getPlotItem().buttonsHidden = True
         self.plot_widget.setAntialiasing(True)
-        self.plot_widget.sizeHint = lambda: QtCore.QSize(500, 500)
+        self.plot_widget.sizeHint = lambda: QSize(500, 500)
+        scene = self.plot_widget.scene()
+        self._create_drag_tooltip(scene)
+
 
         self.replot = self.plot_widget.replot
         ScaleScatterPlotData.__init__(self)
         self.density_img = None
         self.scatterplot_item = None
         self.scatterplot_item_sel = None
+        self.reg_line_item = None
 
         self.labels = []
 
@@ -504,8 +527,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             "missing_shape",
             "Points with undefined '{}' are shown as crossed circles")
         self.shown_attribute_indices = []
-        self.shown_x = ""
-        self.shown_y = ""
+        self.shown_x = self.shown_y = None
         self.pen_colors = self.brush_colors = None
 
         self.valid_data = None  # np.ndarray
@@ -536,6 +558,39 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self._tooltip_delegate = HelpEventDelegate(self.help_event)
         self.plot_widget.scene().installEventFilter(self._tooltip_delegate)
 
+    def _create_drag_tooltip(self, scene):
+        tip_parts = [
+            (Qt.ShiftModifier, "Shift: Add group"),
+            (Qt.ShiftModifier + Qt.ControlModifier,
+             "Shift-{}: Append to group".
+             format("Cmd" if sys.platform == "darwin" else "Ctrl")),
+            (Qt.AltModifier, "Alt: Remove")
+        ]
+        all_parts = ", ".join(part for _, part in tip_parts)
+        self.tiptexts = {
+            int(modifier): all_parts.replace(part, "<b>{}</b>".format(part))
+            for modifier, part in tip_parts
+        }
+        self.tiptexts[0] = all_parts
+
+        self.tip_textitem = text = QGraphicsTextItem()
+        # Set to the longest text
+        text.setHtml(self.tiptexts[Qt.ShiftModifier + Qt.ControlModifier])
+        text.setPos(4, 2)
+        r = text.boundingRect()
+        rect = QGraphicsRectItem(0, 0, r.width() + 8, r.height() + 4)
+        rect.setBrush(QColor(224, 224, 224, 212))
+        rect.setPen(QPen(Qt.NoPen))
+        self.update_tooltip(Qt.NoModifier)
+
+        scene.drag_tooltip = scene.createItemGroup([rect, text])
+        scene.drag_tooltip.hide()
+
+    def update_tooltip(self, modifiers):
+        modifiers &= Qt.ShiftModifier + Qt.ControlModifier + Qt.AltModifier
+        text = self.tiptexts.get(int(modifiers), self.tiptexts[0])
+        self.tip_textitem.setHtml(text)
+
     def new_data(self, data, subset_data=None, **args):
         self.plot_widget.clear()
         self.remove_legend()
@@ -543,6 +598,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.density_img = None
         self.scatterplot_item = None
         self.scatterplot_item_sel = None
+        self.reg_line_item = None
         self.labels = []
         self.selection = None
         self.valid_data = None
@@ -562,6 +618,9 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if self.scatterplot_item_sel:
             self.plot_widget.removeItem(self.scatterplot_item_sel)
             self.scatterplot_item_sel = None
+        if self.reg_line_item:
+            self.plot_widget.removeItem(self.reg_line_item)
+            self.reg_line_item = None
         for label in self.labels:
             self.plot_widget.removeItem(label)
         self.labels = []
@@ -573,22 +632,21 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.master.Information.missing_coords.clear()
         self._clear_plot_widget()
 
-        self.shown_x = attr_x
-        self.shown_y = attr_y
+        self.shown_x, self.shown_y = attr_x, attr_y
 
-        if self.scaled_data is None or not len(self.scaled_data):
+        if self.jittered_data is None or not len(self.jittered_data):
             self.valid_data = None
         else:
-            index_x = self.data_domain.index(attr_x)
-            index_y = self.data_domain.index(attr_y)
-            self.valid_data = self.get_valid_list([index_x, index_y],
-                                                  also_class_if_exists=False)
+            index_x = self.domain.index(attr_x)
+            index_y = self.domain.index(attr_y)
+            self.valid_data = self.get_valid_list([index_x, index_y])
             if not np.any(self.valid_data):
                 self.valid_data = None
         if self.valid_data is None:
             self.selection = None
             self.n_points = 0
-            self.master.Warning.missing_coords(self.shown_x, self.shown_y)
+            self.master.Warning.missing_coords(
+                self.shown_x.name, self.shown_y.name)
             return
 
         x_data, y_data = self.get_xy_data_positions(
@@ -608,7 +666,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         for axis, name, index in (("bottom", attr_x, index_x),
                                   ("left", attr_y, index_y)):
             self.set_axis_title(axis, name)
-            var = self.data_domain[index]
+            var = self.domain[index]
             if var.is_discrete:
                 self.set_labels(axis, get_variable_values_sorted(var))
             else:
@@ -628,7 +686,8 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
         data_indices = np.flatnonzero(self.valid_data)
         if len(data_indices) != self.original_data.shape[1]:
-            self.master.Information.missing_coords(self.shown_x, self.shown_y)
+            self.master.Information.missing_coords(
+                self.shown_x.name, self.shown_y.name)
 
         self.scatterplot_item = ScatterPlotItem(
             x=x_data, y=y_data, data=data_indices,
@@ -645,28 +704,44 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.scatterplot_item.selected_points = []
         self.scatterplot_item.sigClicked.connect(self.select_by_click)
 
+        self.draw_regression_line(x_data, y_data, min_x, max_x)
         self.update_labels()
         self.make_legend()
         self.plot_widget.replot()
 
+    def draw_regression_line(self, x_data, y_data, min_x, max_x):
+        if self.show_reg_line and self.can_draw_regresssion_line():
+            slope, intercept, _, _, _ = linregress(x_data, y_data)
+            start_y = min_x * slope + intercept
+            end_y = max_x * slope + intercept
+            angle = np.degrees(np.arctan((end_y - start_y) / (max_x - min_x)))
+            rotate = ((angle + 45) % 180) - 45 > 90
+            color = QColor("#505050")
+            l_opts = dict(color=color, position=abs(int(rotate) - 0.85),
+                          rotateAxis=(1, 0), movable=True)
+            self.reg_line_item = InfiniteLine(
+                pos=QPointF(min_x, start_y), pen=pg.mkPen(color=color, width=1),
+                angle=angle, label="r = {:.2f}".format(slope), labelOpts=l_opts)
+            if rotate:
+                self.reg_line_item.label.angle = 180
+                self.reg_line_item.label.updateTransform()
+            self.plot_widget.addItem(self.reg_line_item)
+
     def can_draw_density(self):
-        if self.data_domain is None:
-            return False
-        discrete_color = False
-        attr_color = self.attr_color
-        if attr_color != "" and attr_color != "(Same color)":
-            color_var = self.data_domain[attr_color]
-            discrete_color = color_var.is_discrete
-        continuous_x = False
-        continuous_y = False
-        if self.shown_x and self.shown_y:
-            continuous_x = self.data_domain[self.shown_x].is_continuous
-            continuous_y = self.data_domain[self.shown_y].is_continuous
-        return discrete_color and continuous_x and continuous_y
+        return self.domain is not None and \
+            self.attr_color is not None and \
+            self.attr_color.is_discrete and \
+            self.shown_x.is_continuous and \
+            self.shown_y.is_continuous
 
     def should_draw_density(self):
         return self.class_density and self.n_points > 1 and self.can_draw_density()
 
+    def can_draw_regresssion_line(self):
+        return self.domain is not None and \
+               self.shown_x.is_continuous and \
+               self.shown_y.is_continuous
+
     def set_labels(self, axis, labels):
         axis = self.plot_widget.getAxis(axis)
         if labels:
@@ -679,21 +754,20 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.plot_widget.setLabel(axis=axis, text=title)
 
     def get_size_index(self):
-        size_index = -1
-        attr_size = self.attr_size
-        if attr_size != "" and attr_size != "(Same size)":
-            size_index = self.data_domain.index(attr_size)
-        return size_index
+        if self.attr_size is None:
+            return -1
+        return self.domain.index(self.attr_size)
 
     def compute_sizes(self):
         self.master.Information.missing_size.clear()
         size_index = self.get_size_index()
         if size_index == -1:
-            size_data = np.full((self.n_points,), self.point_width)
+            size_data = np.full((self.n_points,), self.point_width,
+                                dtype=float)
         else:
             size_data = \
                 self.MinShapeSize + \
-                self.no_jittering_scaled_data[size_index, self.valid_data] * \
+                self.scaled_data[size_index, self.valid_data] * \
                 self.point_width
         nans = np.isnan(size_data)
         if np.any(nans):
@@ -710,18 +784,15 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
     update_point_size = update_sizes
 
     def get_color_index(self):
-        color_index = -1
-        attr_color = self.attr_color
-        if attr_color != "" and attr_color != "(Same color)":
-            color_index = self.data_domain.index(attr_color)
-            color_var = self.data_domain[attr_color]
-            colors = color_var.colors
-            if color_var.is_discrete:
-                self.discrete_palette = ColorPaletteGenerator(
-                    number_of_colors=len(colors), rgb_colors=colors)
-            else:
-                self.continuous_palette = ContinuousPaletteGenerator(*colors)
-        return color_index
+        if self.attr_color is None:
+            return -1
+        colors = self.attr_color.colors
+        if self.attr_color.is_discrete:
+            self.discrete_palette = ColorPaletteGenerator(
+                number_of_colors=len(colors), rgb_colors=colors)
+        else:
+            self.continuous_palette = ContinuousPaletteGenerator(*colors)
+        return self.domain.index(self.attr_color)
 
     def compute_colors_sel(self, keep_colors=False):
         if not keep_colors:
@@ -732,12 +803,24 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             p.setCosmetic(True)
             return p
 
-        pens = [QPen(Qt.NoPen),
-                make_pen(QColor(255, 190, 0, 255), SELECTION_WIDTH + 1.)]
+        nopen = QPen(Qt.NoPen)
         if self.selection is not None:
+            sels = np.max(self.selection)
+            if sels == 1:
+                pens = [nopen,
+                        make_pen(QColor(255, 190, 0, 255),
+                                 SELECTION_WIDTH + 1.)]
+            else:
+                # Start with the first color so that the colors of the
+                # additional attribute in annotation (which start with 0,
+                # unselected) will match these colors
+                palette = ColorPaletteGenerator(number_of_colors=sels + 1)
+                pens = [nopen] + \
+                       [make_pen(palette[i + 1], SELECTION_WIDTH + 1.)
+                        for i in range(sels)]
             pen = [pens[a] for a in self.selection[self.valid_data]]
         else:
-            pen = [pens[0]] * self.n_points
+            pen = [nopen] * self.n_points
         brush = [QBrush(QColor(255, 255, 255, 0))] * self.n_points
         return pen, brush
 
@@ -754,7 +837,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         subset = None
         if self.subset_indices:
             subset = np.array([ex.id in self.subset_indices
-                               for ex in self.raw_data[self.valid_data]])
+                               for ex in self.data[self.valid_data]])
 
         if color_index == -1:  # same color
             color = self.plot_widget.palette().color(OWPalette.Data)
@@ -769,7 +852,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             return pen, brush
 
         c_data = self.original_data[color_index, self.valid_data]
-        if self.data_domain[color_index].is_continuous:
+        if self.domain[color_index].is_continuous:
             if self.pen_colors is None:
                 self.scale = DiscretizedScale(np.nanmin(c_data), np.nanmax(c_data))
                 c_data -= self.scale.offset
@@ -781,8 +864,9 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                 self.pen_colors = palette.getRGB(c_data)
                 self.brush_colors = np.hstack(
                     [self.pen_colors,
-                     np.full((self.n_points, 1), self.alpha_value)])
-                self.pen_colors *= 100 // self.DarkerValue
+                     np.full((self.n_points, 1), self.alpha_value, dtype=int)])
+                self.pen_colors *= 100
+                self.pen_colors //= self.DarkerValue
                 self.pen_colors = [make_pen(QColor(*col), 1.5)
                                    for col in self.pen_colors.tolist()]
             if subset is not None:
@@ -822,6 +906,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         return pen, brush
 
     def update_colors(self, keep_colors=False):
+        self.master.update_colors()
         if self.scatterplot_item:
             pen_data, brush_data = self.compute_colors(keep_colors)
             pen_data_sel, brush_data_sel = self.compute_colors_sel(keep_colors)
@@ -847,15 +932,15 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             self.labels.append(ti)
 
     def update_labels(self):
-        if not self.attr_label or \
+        if self.attr_label is None or \
                 self.label_only_selected and self.selection is None:
             for label in self.labels:
                 label.setText("")
             return
         if not self.labels:
             self.create_labels()
-        label_column = self.raw_data.get_column_view(self.attr_label)[0]
-        formatter = self.raw_data.domain[self.attr_label].str_val
+        label_column = self.data.get_column_view(self.attr_label)[0]
+        formatter = self.attr_label.str_val
         label_data = map(formatter, label_column)
         black = pg.mkColor(0, 0, 0)
         if self.label_only_selected:
@@ -867,13 +952,10 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                 label.setText(text, black)
 
     def get_shape_index(self):
-        shape_index = -1
-        attr_shape = self.attr_shape
-        if attr_shape and attr_shape != "(Same shape)" and \
-                len(self.data_domain[attr_shape].values) <= \
-                len(self.CurveSymbols):
-            shape_index = self.data_domain.index(attr_shape)
-        return shape_index
+        if self.attr_shape is None or \
+                len(self.attr_shape.values) > len(self.CurveSymbols):
+            return -1
+        return self.domain.index(self.attr_shape)
 
     def compute_symbols(self):
         self.master.Information.missing_shape.clear()
@@ -931,7 +1013,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         color_index = self.get_color_index()
         if color_index == -1:
             return
-        color_var = self.data_domain[color_index]
+        color_var = self.domain[color_index]
         use_shape = self.get_shape_index() == color_index
         if color_var.is_discrete:
             if not self.legend:
@@ -960,7 +1042,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             return
         if not self.legend:
             self.create_legend()
-        shape_var = self.data_domain[shape_index]
+        shape_var = self.domain[shape_index]
         color = self.plot_widget.palette().color(OWPalette.Data)
         pen = QPen(color.darker(self.DarkerValue))
         color.setAlpha(self.alpha_value)
@@ -1005,19 +1087,25 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def select(self, points):
         # noinspection PyArgumentList
-        if self.raw_data is None:
+        if self.data is None:
             return
-        keys = QApplication.keyboardModifiers()
-        if self.selection is None or not keys & (
-                Qt.ShiftModifier + Qt.ControlModifier + Qt.AltModifier):
-            self.selection = np.full(len(self.raw_data), False, dtype=np.bool)
+        if self.selection is None:
+            self.selection = np.zeros(len(self.data), dtype=np.uint8)
         indices = [p.data() for p in points]
+        keys = QApplication.keyboardModifiers()
+        # Remove from selection
         if keys & Qt.AltModifier:
-            self.selection[indices] = False
-        elif keys & Qt.ControlModifier:
-            self.selection[indices] = ~self.selection[indices]
-        else:  # Handle shift and no modifiers
-            self.selection[indices] = True
+            self.selection[indices] = 0
+        # Append to the last group
+        elif keys & Qt.ShiftModifier and keys & Qt.ControlModifier:
+            self.selection[indices] = np.max(self.selection)
+        # Create a new group
+        elif keys & Qt.ShiftModifier:
+            self.selection[indices] = np.max(self.selection) + 1
+        # No modifiers: new selection
+        else:
+            self.selection = np.zeros(len(self.data), dtype=np.uint8)
+            self.selection[indices] = 1
         self.update_colors(keep_colors=True)
         if self.label_only_selected:
             self.update_labels()
@@ -1025,7 +1113,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def get_selection(self):
         if self.selection is None:
-            return np.array([], dtype=int)
+            return np.array([], dtype=np.uint8)
         else:
             return np.flatnonzero(self.selection)
 
@@ -1047,22 +1135,22 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                 index = p.data()
                 text += "Attributes:\n"
                 if self.tooltip_shows_all and \
-                        len(self.data_domain.attributes) < 30:
+                        len(self.domain.attributes) < 30:
                     text += "".join(
                         '   {} = {}\n'.format(attr.name,
-                                              self.raw_data[index][attr])
-                        for attr in self.data_domain.attributes)
+                                              self.data[index][attr])
+                        for attr in self.domain.attributes)
                 else:
                     text += '   {} = {}\n   {} = {}\n'.format(
-                        self.shown_x, self.raw_data[index][self.shown_x],
-                        self.shown_y, self.raw_data[index][self.shown_y])
+                        self.shown_x, self.data[index][self.shown_x],
+                        self.shown_y, self.data[index][self.shown_y])
                     if self.tooltip_shows_all:
                         text += "   ... and {} others\n\n".format(
-                            len(self.data_domain.attributes) - 2)
-                if self.data_domain.class_var:
+                            len(self.domain.attributes) - 2)
+                if self.domain.class_var:
                     text += 'Class:\n   {} = {}\n'.format(
-                        self.data_domain.class_var.name,
-                        self.raw_data[index][self.raw_data.domain.class_var])
+                        self.domain.class_var.name,
+                        self.data[index][self.data.domain.class_var])
                 if i < len(points) - 1:
                     text += '------------------\n'
 
